package cheddar.aadl;

import org.osate.aadl2.ClassifierValue;
import org.osate.aadl2.ComponentCategory;
import org.osate.aadl2.ComponentClassifier;
import org.osate.aadl2.NamedElement;
import org.osate.aadl2.Property;
import org.osate.aadl2.PropertyExpression;
import org.osate.aadl2.instance.ComponentInstance;
import org.osate.xtext.aadl2.properties.util.GetProperties;
import org.osate.xtext.aadl2.properties.util.ModelingProperties;

import cheddar.adl.Policies;
import cheddar.adl.Resources_Type;
import cheddar.adl.Schedulers_Type;
import cheddar.adl.Tasks_Type;
import cheddar.adl.Time_Triggered_Communication_Timing_Property_Type;

public class Properties_With_Default_Value {


	public static ComponentClassifier getImplementedAs(final NamedElement ne) {

		PropertyExpression pv = null;
		Property referenceProcessor = null;
		try {

			referenceProcessor = GetProperties.lookupPropertyDefinition(ne, ModelingProperties._NAME, "Implemented_As");
			if (referenceProcessor == null) {
				return null;
			}
		}
		catch (Exception e) {}

		try {
			pv = ne.getSimplePropertyValue(referenceProcessor);
			if (pv == null) {
				return null;
			}
		}
		catch (Exception e) {}
		return (ComponentClassifier) ((ClassifierValue) pv).getClassifier();
	}

	public static String getBoundProcess(ComponentInstance component) {
		String process_name = "";

		ComponentInstance co = component.getContainingComponentInstance();
		while (process_name.equals("") && co != null) {
				if (co.getCategory() == ComponentCategory.PROCESS) {
				process_name = co.getName();
				}
			co = co.getContainingComponentInstance();
			}

		return process_name;
	}

	public static String getBoundProcessor(ComponentInstance component) {
		String processor_name = "";

		ComponentInstance co = GetProperties.getBoundProcessor(component);
		if (co != null) {
				processor_name = co.getName();
		}

		return processor_name;
	}

	public static Time_Triggered_Communication_Timing_Property_Type getConnectionTiming(final NamedElement pc)
	{
		String l = GetProperties.getConnectionTiming(pc).getName();
		Time_Triggered_Communication_Timing_Property_Type protocol = Time_Triggered_Communication_Timing_Property_Type.Sampled_Timing;
		l=l.toLowerCase();
		if (l.equals("sampled")) {
			protocol = Time_Triggered_Communication_Timing_Property_Type.Sampled_Timing;
		} else
			if (l.equals("immediate")) {
				protocol = Time_Triggered_Communication_Timing_Property_Type.Immediate_Timing;
		} else if (l.equals("delayed")) {
			protocol = Time_Triggered_Communication_Timing_Property_Type.Delayed_Timing;
	}

		return protocol;
	}

	public static Policies getPOSIX_Scheduling_Policy(final NamedElement ne) {
		Policies policy = Policies.Sched_Fifo;
		try {
			String s=Cheddar_Properties_Util.getPOSIX_Scheduling_Policy(ne).getName();
			if (s != null) {
				policy = Policies.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return policy;
	}

	public static Resources_Type getConcurrencyControlProtocol(final NamedElement ne) {
		Resources_Type rt = Resources_Type.No_Protocol;
		try {
			String s = GetProperties.getConcurrencyControlProtocol(ne);
			if (s != null) {
				if (s.toLowerCase().equals("priority_ceiling")) {
					return Resources_Type.Priority_Ceiling_Protocol;
				} else {
					rt = Resources_Type.fromString(s.toLowerCase());
				}
			}

		} catch (Exception e) {
		}
		return rt;
	}

	public static Tasks_Type getDispatchProtocol(final NamedElement ne) {
		Tasks_Type tt = Tasks_Type.Periodic_Type;
		try {
			String s = GetProperties.getDispatchProtocol(ne).getName();
			if (s != null) {
				tt = Tasks_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return tt;
	}

	public static Integer getDispatch_Jitter(final NamedElement ne) {
		Integer jitter = 0;
		try {
			jitter = (int) Cheddar_Properties_Util.getDispatch_JitterinMilliSec(ne);
		} catch (Exception e) {
		}
		return jitter;
	}

	public static Integer getDispatch_Absolute_Time(final NamedElement ne) {
		Integer start_time = 0;
		try {
			start_time = (int) Cheddar_Properties_Util.getDispatch_Absolute_TimeinMilliSec(ne);
		} catch (Exception e) {
		}
		return start_time;
	}

	public static Integer getExecutionTime(final NamedElement ne) {
		Integer execution_time = 0;
		try {
			execution_time = (int) GetProperties.getMaximumComputeExecutionTimeinMs(ne);
		} catch (Exception e) {
		}
		return execution_time;
	}

	public static Integer getDeadline(final NamedElement ne) {
		Integer deadline = 0;
		try {
			deadline = (int) GetProperties.getDeadlineinMilliSec(ne);
		} catch (Exception e) {
		}
		return deadline;
	}

	public static Integer getPeriod(final NamedElement ne) {
		Integer period = 0;
		try {
			period = (int) GetProperties.getPeriodinMS(ne);
		} catch (Exception e) {
		}
		return period;
	}
	public static Integer getPriority(final NamedElement ne) {
		Integer priority = 0;
		try {
			priority = (int) Cheddar_Properties_Util.getFixedPriority(ne);
			priority = (int) GetProperties.getPriority(ne, 0);
		} catch (Exception e) {
		}
		return priority;
	}

	public static Integer getScheduler_Quantum(final NamedElement ne) {
		Integer quantum = 0;
	try {
			quantum = Cheddar_Properties_Util.getScheduler_Quantum(ne);
		} catch (Exception e) {
		}
		return quantum;
	}

	public static String getSource_Text(final NamedElement ne) {
		String source_text = "";
		try {
			source_text = Cheddar_Properties_Util.getSource_Text(ne);
			if (source_text == null) {
				source_text="";
			}
		} catch (Exception e) {
		}
		return source_text;
	}

	public static Schedulers_Type getSchedulingProtocol(ComponentInstance ne) {
		Schedulers_Type protocol = Schedulers_Type.No_Scheduling_Protocol;
		try {
			String s = GetProperties.getSchedulingProtocol(ne);
			if (s != null) {
				protocol = Schedulers_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return protocol;
	}

	public static boolean getPreemptive_Scheduler(final NamedElement ne) {
	boolean preemptive = true;
	try {
			preemptive = Cheddar_Properties_Util.getPreemptive_Scheduler(ne);
	} catch (Exception e) {
	}
	return preemptive;
	}

}
